/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.agent;

import com.redhat.thermostat.backend.BackendConfiguration;
import com.redhat.thermostat.backend.BackendConfigurationRegistry;
import com.redhat.thermostat.testutils.StubBundleContext;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.cm.Configuration;
import org.osgi.service.cm.ConfigurationAdmin;

import java.util.Hashtable;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class BackendConfigurationListenerTest {

    private BundleContext context;
    private BackendConfiguration configuration;
    private BackendConfigurationRegistry registry;
    private ConfigurationAdmin configurationAdmin;
    private Configuration mockConfig;

    @Before
    public void setUp() throws Exception {
        configuration = mock(BackendConfiguration.class);
        when(configuration.isEnabled()).thenReturn(true);
        when(configuration.getLocation()).thenReturn("/foo/bar/test-plugin-1.99.12-SNAPSHOT.jar");
        when(configuration.getPluginId()).thenReturn("test-plugin");
        context = new StubBundleContext();
        registry = new BackendConfigurationRegistry(context);
        BackendConfigurationListener listener = new BackendConfigurationListener();
        registry.addActionListener(listener);
        configurationAdmin = mock(ConfigurationAdmin.class);
        listener.bindConfigurationAdmin(configurationAdmin);
        mockConfig = mock(Configuration.class);
        when(configurationAdmin.getConfiguration(anyString())).thenReturn(mockConfig);
        registry.start();
    }

    @Test
    public void testConfigurationRegistered() throws Exception {
        context.registerService(BackendConfiguration.class, configuration, null);
        verify(configuration).isEnabled();
        verify(configurationAdmin).getConfiguration(eq("test-plugin"));
        verify(mockConfig).setBundleLocation(eq("/foo/bar/test-plugin-1.99.12-SNAPSHOT.jar"));
        verify(mockConfig).update(any(Hashtable.class));
    }

    @Test
    public void testConfigurationUnregistered() throws Exception {
        ServiceRegistration reg = context.registerService(BackendConfiguration.class, configuration, null);
        verify(configuration).isEnabled();
        verify(configurationAdmin).getConfiguration(eq("test-plugin"));
        verify(mockConfig).setBundleLocation(eq("/foo/bar/test-plugin-1.99.12-SNAPSHOT.jar"));
        verify(mockConfig).update(any(Hashtable.class));
        reg.unregister();
        verify(mockConfig).delete();
    }

    @After
    public void tearDown() {
        registry.stop();
    }
}
